local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")

function Capitalist:AUCTION_HOUSE_SHOW()
	self:DebugEntry("AUCTION_HOUSE_SHOW")
	
	-- REVIEW: Workaround for Blizzard_AuctionUI.lua 1458
	-- It appears to be an issue introduced in 3.3 or 4.01 that the
	-- if duration combo box is not yet created, then
	-- CalculateAuctionDeposit blows up.
	AuctionFrameTab_OnClick(AuctionFrameTab3)
	AuctionFrameTab_OnClick(AuctionFrameTab1)

	if (self.db.profile[realm] and
		self.db.profile[realm][character]) then

		for profession, professionData in pairs(self.db.profile[realm][character]) do
			self:UpdateRestockStack(profession)
		end
	end

	self:UpdateAuctionUI()
	self:UpdateComponentUI()
	self:DebugExit("AUCTION_HOUSE_SHOW")
end

function Capitalist:AUCTION_OWNED_LIST_UPDATE()
	self:DebugEntry("AUCTION_OWNED_LIST_UPDATE")
	self:UpdateAuctionUI()
	self:DebugExit("AUCTION_OWNED_LIST_UPDATE")
end

function Capitalist:CalculateCollectionDifferences(collectionOld, collectionNew)
	local collectionDelta = { }
	for k,v in pairs(collectionNew) do
		local deltaK = collectionNew[k] - (collectionOld[k] or 0)
		if (deltaK ~= 0) then
			collectionDelta[k] = deltaK
		end
	end
	for k,v in pairs(collectionOld) do
	    if (not collectionNew[k]) then
			collectionDelta[k] = - collectionOld[k]
		end
	end
	return collectionDelta
end

function Capitalist:RecordSalvageInput(...)
	self:DebugEntry("RecordSalvageInput")

	self:DebugMisc("Input:")
	self:DebugMisc("  "..self.spellTarget)
	local inventorySpellTerm = self:BuildInventoryCollection()
	local inventorySpellDelta = self:CalculateCollectionDifferences(self.inventorySpellInit, inventorySpellTerm)
	local spellTargetName = GetItemInfo(self.spellTarget)
	local spellTargetNumConsumed = 0
	for k,v in pairs(inventorySpellDelta) do			
		--print(spellTargetName.." vs. "..k)
		if (spellTargetName == k) then
			self:DebugMisc(string.format("  %d x %s", -v, k))
			--
			print(string.format("  %d x %s", -v, k))
			spellTargetNumConsumed = -v
		end
	end
	print("spellTarget="..self.spellTarget)
	print("spellTargetName="..spellTargetName)
	print("spellTargetNumConsumed="..spellTargetNumConsumed)

	if (self.db.profile.enableSalvageTracking) then
		local recipeLink = self.spellLinkLast
		if (recipeLink and
			self.db.profile[realm] and
			self.db.profile[realm][character] and
			self.db.profile[realm][character][self.spellNameLastStarted]
			) then

--			print(self.db.profile[realm])
--			print(self.db.profile[realm][character])
--			print(self.db.profile[realm][character][self.spellNameLastStarted])
--			print(self.db.profile[realm][character][self.spellNameLastStarted].numConsume)
--			print(self.db.profile[realm][character][self.spellNameLastStarted]["numConsume"])

			local spellData = self.db.profile[realm][character][self.spellNameLastStarted]

			-- add/update the consumption
			if (spellTargetNumConsumed > 0) then
				if (spellData.numConsume ~= spellTargetNumConsumed) then
					-- TODO
					print(self.spellNameLastStarted)
					if (spellData.numConsume) then
						print("WANT TO UPDATE FROM: "..spellData.numConsume)
					end
					print("WANT TO UPDATE TO: "..spellTargetNumConsumed)
					spellData.numConsume = spellTargetNumConsumed

else
print("MATCHES: "..spellTargetNumConsumed)
				end
			end
		end
	end

	self.inventorySpellInit = self:BuildInventoryCollection()

	if (spellTargetNumConsumed > 0) then
--		self.spellTarget = nil
--		self.spellNameLastStarted = nil
	end

	self:DebugExit("RecordSalvageInput")
end

-- REVIEW: remove unused event handler
function Capitalist:LOOT_CLOSED(...)
	self:DebugEntry("LOOT_CLOSED")

	if (self.spellLinkLast and self.spellTarget) then
		self:DebugMisc("BAGIMPACT LC INIT: "..self.spellLinkLast)

		--self:RecordSalvageInput()
	
		self:DebugMisc("BAGIMPACT LC TERM: "..self.spellLinkLast)
	end

	self:DebugExit("LOOT_CLOSED")
end

function Capitalist:LOOT_OPENED(...)
	self:DebugEntry("LOOT_OPENED")

	--print("qqq")
	--print(self.spellLinkLast)
	--print(self.spellTarget)
	--print("rrr")
print("salvage tracking prior")
print(self.spellLinkLast)
print(self.spellTarget)
	if (self.spellLinkLast and self.spellTarget) then
		self:DebugMisc("BAGIMPACT LO INIT: "..self.spellLinkLast)

--		self:DebugMisc("Input:")
--		self:DebugMisc("  "..self.spellTarget)
--		local inventorySpellTerm = self.db.profile.storage[realm][character].inventory
--		local inventorySpellDelta = self:CalculateCollectionDifferences(self.inventorySpellInit, inventorySpellTerm)
--		local spellTargetName = GetItemInfo(self.spellTarget)
--		local spellTargetNumConsumed = 0
--		for k,v in pairs(inventorySpellDelta) do			
--			print(spellTargetName.." vs. "..k)
--			if (spellTargetName == k) then
--				self:DebugMisc(string.format("  %d x %s", -v, k))
--				spellTargetNumConsumed = -v
--			end
--		end
--		print("spellTarget="..self.spellTarget)
--		print("spellTargetName="..spellTargetName)
--		print("spellTargetNumConsumed="..spellTargetNumConsumed)
		self:RecordSalvageInput()

		local lootFound
		local lootCollection = {}
		for i = 1, GetNumLootItems() do
			if (LootSlotIsItem(i)) then
				--local lootIcon, lootName, lootQuantity, rarity, locked = GetLootSlotInfo(i)
				local _, _, lootQuantity, _, _ = GetLootSlotInfo(i)
				local lootLink = GetLootSlotLink(i)

				local lootLinkGeneral = self:LinkStripSuffix(self:LinkStripUniqueId(lootLink))

				--local itemIdString = string.match(lootLinkGeneral, "Hitem:(%d+)")
				--local itemIdNumber = tonumber(itemIdString)

				if (lootLinkGeneral) then
					lootCollection[lootLinkGeneral] = (lootCollection[lootLinkGeneral] or 0) + lootQuantity
					lootFound = true
				end
			end
		end

		self:DebugMisc("Output:")
		for k,v in pairs(lootCollection) do
			self:DebugMisc(string.format("  %d x %s", v, k))
		end

		if (self.db.profile.enableSalvageTracking) then
print("salvage tracking 1")

			--local recipeLink = GetSpellLink(self.spellLinkLast)
			local recipeLink = self.spellLinkLast
			if (lootFound and recipeLink) then
				if (self.db.profile[realm] and self.db.profile[realm][character]) then
					if (not self.db.profile[realm][character][self.spellNameLastStarted]) then
						self.db.profile[realm][character][self.spellNameLastStarted] = { }
					end
				end

				local spellData = self.db.profile[realm][character][self.spellNameLastStarted]

				-- add/update the skill link
				if (spellData.recipeLink ~= recipeLink) then
					spellData.recipeLink = recipeLink
				end

				-- add/update the consumption
--				if (spellData.numConsume ~= spellTargetNumConsumed) then
--					spellData.numConsume = spellTargetNumConsumed
--				end

				-- generalize the spell target item (remove the unique id for the salvagee)
				local spellTargetGeneral = self:LinkStripSuffix(self:LinkStripUniqueId(self.spellTarget))

				-- create collection for target item
				if (not spellData[spellTargetGeneral]) then
					spellData[spellTargetGeneral] = { }
				end

				-- record the use increment
				spellData[spellTargetGeneral].numCraft = 1 + (spellData[spellTargetGeneral].numCraft or 0)

				-- add/update the required amount of the target item
				if (spellData[spellTargetGeneral].numConsumed ~= numConsumed) then
					spellData[spellTargetGeneral].numConsumed = numConsumed
				end

				-- add/update resources tallies
				if (not spellData[spellTargetGeneral].resourceTally) then
					spellData[spellTargetGeneral].resourceTally = { }
				end
				for k,v in pairs(lootCollection) do
					spellData[spellTargetGeneral].resourceTally[k] = v + (spellData[spellTargetGeneral].resourceTally[k] or 0)
				end

				self:UpdateStatistics()
				AceConfigRegistry:NotifyChange("Capitalist")
			end
		end
	
		self:DebugMisc("BAGIMPACT LO TERM: "..self.spellLinkLast)

		--self.spellTarget = nil
	end

	self:DebugExit("LOOT_OPENED")
end

function Capitalist:AddContainerContentsToCollection(bag, collection, includeCraftingOnly)
	--self:DebugEntry("AddContainerContentsToCollection")
	--self:DebugMisc(bag)
	--self:DebugMisc(collection)
	--self:DebugMisc(includeCraftingOnly)

	for slot = 1, GetContainerNumSlots(bag) do
		local link = GetContainerItemLink(bag, slot)
		if (link) then
			local name = link:match("%[(.*)%]")

			-- REVIEW: Correction for two glyphs named "Glyph of Death Coil"
			local itemIdString = string.match(link, "Hitem:(%d+)")
			local itemIdNumber = tonumber(itemIdString)
			-- item 45804, spell 64266 - DK
			if (itemIdNumber == 45804) then
				name = string.gsub(name, "Death Coil", "Death Coil <Death Knight>")
			end
			-- item 42457, spell 57261 - Warlock
			if (itemIdNumber == 42457) then
				name = string.gsub(name, "Death Coil", "Death Coil <Warlock>")
			end

			local exclude = false
			if (includeCraftingOnly) then
				if (self.db.profile.components and self.db.profile.components[name]) then
					-- component
				elseif (self.list and self.list[name]) then
					-- product
				else
					exclude = true
				end
			end

			if (not exclude) then
				local _, count = GetContainerItemInfo(bag, slot)
				collection[name] = count + (collection[name] or 0)
			end
		end
	end

	--self:DebugExit("AddContainerContentsToCollection")
end

function Capitalist:BuildInventoryCollection(includeCraftingOnly)
	local collection = {}
	for bag = 0, 4 do
		self:AddContainerContentsToCollection(bag, collection, includeCraftingOnly)
	end
	return collection
end

function Capitalist:BAG_UPDATE(...)
	self:DebugEntry("BAG_UPDATE")

	local event, arg1 = ...;
	self:DebugMisc(arg1)

	-- save all products and components in that players inventory
	local itemsInInv = self.db.profile.storage[realm][character].inventory
	for k, v in pairs(itemsInInv) do
		itemsInInv[k] = nil
	end
	
	for bag = 0, 4 do
		self:AddContainerContentsToCollection(bag, itemsInInv, true)
	end

	-- review: the second half of this clause was triggered by small heavenly shard 'use'; consider using this to capture such conversions
	if (self.spellTarget and self.inventorySpellInit) then
		self:RecordSalvageInput()
	end

	if (self.mode == "posting") then
		if (self.PostingPostActive) then
			Capitalist:PostCallback(true)
		end
	end

	self:DebugExit("BAG_UPDATE")
end

function Capitalist:BANKFRAME_CLOSED(...)
	self:DebugEntry("BANKFRAME_CLOSED")

	if (self.bankFrameOpen) then
		self.bankFrameOpen = false

		local itemsInBank = self.db.profile.storage[realm][character].bank
		for k, v in pairs(itemsInBank) do
			itemsInBank[k] = nil
		end
	
		local containersToCheck = {-1, 5, 6, 7, 8, 9, 10, 11}
	
		for _, bag in pairs(containersToCheck) do
			self:AddContainerContentsToCollection(bag, itemsInBank, true)
		end
	end

	self:DebugExit("BANKFRAME_CLOSED")
end

function Capitalist:BANKFRAME_OPENED(...)
	self:DebugEntry("BANKFRAME_OPENED")
	self.bankFrameOpen = true
	self:DebugExit("BANKFRAME_OPENED")
end

function Capitalist:CHAT_MSG_LOOT(...)
	self:DebugEntry("CHAT_MSG_LOOT")

	local event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11 = ...;
	self:DebugMisc(arg1)	--You create: [Scroll of Enchant Chest - Powerful Stats].
	self:DebugMisc(arg7)	--0
	self:DebugMisc(arg8)	--0
	self:DebugMisc(arg9)	--(empty string)
	self:DebugMisc(arg10)	--0
	self:DebugMisc(arg11)	--1123

	local staleStatistics = false
	if (arg1:find(L["You create:"])) then

		local itemLink = string.sub(arg1, 13)
		-- -1 to remove the trailing period also
		itemLink = string.sub(itemLink, 0, strlen(itemLink) - 1)

		local productName, productCount = arg1:match("%[(.*)%].*x?(%d+)\.$")
		-- REVIEW: why doesn't the regex above work for the version with no "x5" at the end?
		if (not productName) then
			productName = arg1:match("%[(.*)%]")
			productCount = 1
		end
		self:DebugMisc(productName)
		self:DebugMisc(productCount)
		productCount = tonumber(productCount)

		--self:LinkDissect(itemLink)
		--print(itemLink)
--print(self:LinkStripSuffix(self:LinkStripUniqueId(itemLink)))

		-- REVIEW: Correction for two glyphs named "Glyph of Death Coil"
		local itemIdString = string.match(arg1, "Hitem:(%d+)")
		local itemIdNumber = tonumber(itemIdString)
		-- item 45804, spell 64266 - DK
		if (itemIdNumber == 45804) then
			productName = string.gsub(productName, "Death Coil", "Death Coil <Death Knight>")
		end
		-- item 42457, spell 57261 - Warlock
		if (itemIdNumber == 42457) then
			productName = string.gsub(productName, "Death Coil", "Death Coil <Warlock>")
		end

		if (self.skillCompleted) then
			local profession = self.skillCompleted.profession
			local skillName = self.skillCompleted.skillName

			local numProductOld = self.db.profile[realm][character][profession][skillName].numProduct or 0
			self.db.profile[realm][character][profession][skillName].numProduct = numProductOld + productCount

			local skillItemLink = self.db.profile.professions[profession][skillName].itemLink
			local skillItemId = self:LinkExtractId(skillItemLink)
			if (skillItemId ~= itemIdString) then
			
				-- Exclude enchanting scrolls from 'rareTally' because (a) they don't have rare crafts and
				-- (b) this logic is confused by the difference between the Henchant and the Hitem
				if (not productName:find(L["Enchant "])) then
					print("+++ Unusual craft")
					print(skillItemId)
					print(itemIdString)

					local characterSkillData = self.db.profile[realm][character][profession][skillName]
					if (not characterSkillData.rareTally) then
						characterSkillData.rareTally = {}
					end

					local itemLinkGeneral = self:LinkStripSuffix(self:LinkStripUniqueId(itemLink))
					characterSkillData.rareTally[itemLinkGeneral] = 1 + (characterSkillData.rareTally[itemLinkGeneral] or 0)
				end
			end

			staleStatistics = true

			if (self.list[productName]) then
				if (self.restock and
					self.restock[profession] and
					self.restock[profession][skillName]) then
					self.restock[profession][skillName] = self.restock[profession][skillName] - productCount
					if (self.restock[profession][skillName] <= 0) then
						self.restock[profession][skillName] = nil
					end
				end
			end
		end

		if (staleStatistics) then
			self:UpdateStatistics()
			AceConfigRegistry:NotifyChange("Capitalist")
		end

		self.inTradeskill = nil
	end

	self:DebugExit("CHAT_MSG_LOOT")
end

function Capitalist:CHAT_MSG_SYSTEM(...)
	self:DebugEntry("CHAT_MSG_SYSTEM")

	local event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11 = ...;
	self:DebugMisc(arg1)	--Auction created.

	if (arg1:find(L["Auction created."])) then
		if (self.PostingPostActive) then
			Capitalist:PostCallback(true)
		end
	end

	self:DebugExit("CHAT_MSG_SYSTEM")
end

function Capitalist:MERCHANT_CLOSED(event, addon)
	self:DebugEntry("MERCHANT_CLOSED")
	self.MerchantActive = nil
	self:DebugExit("MERCHANT_CLOSED")
end

function Capitalist:MERCHANT_SHOW(event, addon)
	self:DebugEntry("MERCHANT_SHOW")
	self.MerchantActive = true

	local dataUpdated = false

	local merchant = UnitName("target")

	local outputtedHeader
	for i=1, GetMerchantNumItems() do
		local name, texture, price, quantity, numAvailable, isUsable, extendedCost = GetMerchantItemInfo(i)

		if (name and price and quantity and
			-- only include items of unlimited stock
			(numAvailable == -1) and
			-- TODO: PVP-bought gems
			(extendedCost == nil)) then

			-- REVIEW: In light of Enchanting Vellum now being merchant-supplied, ...
			-- should we relax this requirement so that products can have merchant
			-- pricing set on them?  What impact does this have on the terminology
			-- and the system as a whole?
			--
			-- Would it be simpler to do some form of evaluation that does a cost
			-- comparison and decides that one or the other 'wins', and the other
			-- effectively vanishes from the system entirely?
			--
			-- A "deprecated recipe" model fits for the Enchanting Vellum case,
			-- certainly.
			--
			-- But are there other scenarios where if the price of the components
			-- to craft the product dip low enough that we'd want to craft instead?
			-- If so, that suggests we need to keep both "active" and have some
			-- kind of a dynamic system that reacts to set / market prices for
			-- materials and chooses whether to "vendor acquire" or "produce"
			-- the component depending on pricing, availability, etc.
			--
			-- The latter paragraph sounds exactly like what we want to do for
			-- limited supply merchant resources when the buyPrice from merchant
			-- is effectively a discount (because of AH overpricing) or as an
			-- alternative to lack of component availability to go the craft route.
			--
			-- Sidenote: Auto-purchase or 'recommend' text to the user to hint
			-- that limited supply resources should be obtained, is an interesting
			-- feature idea as well.
			if (self:IsResource(name)) then
				local hadBuyPrice = self:HasResourceCopperBuyPrice(name)
				local buyPricePerUnitNew = price / quantity
				local buyPricePerUnitOld = self:GetResourceCopperBuyPrice(name)

				if ((self.db.profile.enableMerchantCosting) and

					-- Always go with the lowest price we've seen
					(	(not hadBuyPrice) or
						(buyPricePerUnitOld > buyPricePerUnitNew)
					)) then

					if (not outputtedHeader) then
						if (self.db.profile.filters.global.NotifyMerchantCostChanges) then
							self:Print(string.format(L["Merchant %s update:"], merchant))
						end
						outputtedHeader = true
						dataUpdated = true
					end

					-- REVIEW: Revisit all such rounding comments, trimming stale ones.
					-- NOTE: This intentionally does /10000 then *10000 logic
					-- so that rounding/truncation in the stored data is visible
					local newCostString = self:PrettyCurrency(buyPricePerUnitNew)
					local oldCostString = self:PrettyCurrency(buyPricePerUnitOld)

					if (self.db.profile.filters.global.NotifyMerchantCostChanges) then
						self:Print(
							string.format(
								"  "..L["Updating merchant for %s and buy price from %s to %s."],
								self:GetComponentText(name), oldCostString, newCostString
							))
					end
					self:ResourceSetMerchant(name, merchant)
					self:SetResourceCopperBuyPrice(name, buyPricePerUnitNew)
					self:SetResourceCopperCost(name, nil)

					-- When updating the buyPrice for a merchant-supplied resource, if the
					-- buyPrice is below the previous valuation of the component then the
					-- products build from that resource (directly or indirectly) now have
					-- potentially incorrect valuations.  Rather than build up that entire
					-- set of relationships to identify exactly which to update (and potentially
					-- get it wrong), we just clear out all value calculations to force recalc.
					if (self:HasComponentCopperValue(name) and
						(self:GetComponentCopperValue(name) > buyPricePerUnitNew)) then
						self:ClearValues()
					end

				else
					if ((not self:HasResourceCopperBuyPrice(name)) or
						(buyPricePerUnitOld >= buyPricePerUnitNew)) then
						if (merchant and not self:ResourceGetMerchant(name)) then
							if (not outputtedHeader) then
								if (self.db.profile.filters.global.NotifyMerchantCostChanges) then
									self:Print(string.format(L["Merchant %s update:"], merchant))
								end
								outputtedHeader = true
								self:CostMemoryReset()
							end
							if (self.db.profile.filters.global.NotifyMerchantCostChanges) then
								self:Print(string.format("  "..L["Updating merchant for %s."], name))
							end
							self:ResourceSetMerchant(name, merchant)
						end
					end
				end
			end
		end
	end

	if (dataUpdated) then
		self:UpdateComponentValues()
		self:UpdateCombinedValues()
	end

	self:DebugExit("MERCHANT_SHOW")
end

function Capitalist:UI_ERROR_MESSAGE(...)
	self:DebugEntry("UI_ERROR_MESSAGE")

	local event, arg1 = ...;
	self:DebugMisc(arg1)

	if (arg1 == L["Inventory is full."]) then
		self.inTradeskill = nil
	end

	self:DebugExit("UI_ERROR_MESSAGE")
end

function Capitalist:UNIT_SPELLCAST_INTERRUPTED(...)
	self:DebugEntry("UNIT_SPELLCAST_INTERRUPTED")

	local event, arg1, arg2, arg3, arg4, arg5 = ...;
	self:DebugMisc(arg1)
	self:DebugMisc(arg2)
	self:DebugMisc(arg4)

	if (arg1 == "player") then
		self.inTradeskill = nil
		self.inventorySpellInit = nil
		self.spellTarget = nil
		self.spellNameLastStarted = nil
		self.spellLinkLast = nil
	end

	self:DebugExit("UNIT_SPELLCAST_INTERRUPTED")
end

function Capitalist:UNIT_SPELLCAST_START(...)
	self:DebugEntry("UNIT_SPELLCAST_START")

	local event, arg1, arg2, arg3, arg4, arg5 = ...;
	self:DebugMisc(arg1)	-- Unit casting the spell	(player)
	self:DebugMisc(arg2)	-- Spell name				(Captain Rumsey's Lager)
	self:DebugMisc(arg3)	-- Spell rank				()
	self:DebugMisc(arg4)	-- Spell lineID counter		(9)
	self:DebugMisc(arg5)	-- Spell ID					(45695)

	local spellName = arg2
	local spellLink = GetSpellLink(arg5)

	--local name, rank, icon, powerCost, isFunnel, powerType, castingTime, minRange, maxRange = GetSpellInfo(arg5)
	local _, _, _, powerCost, _, _, _, _, _ = GetSpellInfo(arg5)
	
	--print("xxx")
	--print(arg1)
	--print(spellName)
	--print(spellLink)
	--print(powerCost)
	--print("yyy")
	if ((arg1 == "player") and
		spellName and
		spellLink and
		((not powerCost) or (powerCost == 0))) then

		--self.inventorySpellInit = self:CopyCollectionInventory()
		self.inventorySpellInit = self:BuildInventoryCollection()
		self.spellNameLastStarted = spellName
		print("><>"..arg2)
		print("><>"..arg5)
		self.spellLinkLast = spellLink

		print("><>"..spellLink)
		--self:LinkDissect(spellLink)

		--local name, rank, icon, powerCost, isFunnel, powerType, castingTime, minRange, maxRange = GetSpellInfo(arg5)
		--print("-->")
		--print("n")
		--print(name)
		--print("r")
		--print(rank)
		--print("i")
		--print(icon)
		--print("pC")
		--print(powerCost)
		--print("iF")
		--print(isFunnel)
		--print("pT")
		--print(powerType)
		--print("cT")
		--print(castingTime)
		--print("nR")
		--print(minRange)
		--print("xR")
		--print(maxRange)
		--print("--<")
	end
end

-- REVIEW: remove unused event handler
function Capitalist:UNIT_SPELLCAST_STOP(...)
	self:DebugEntry("UNIT_SPELLCAST_STOP")

	local event, arg1, arg2, arg3, arg4, arg5 = ...;
	self:DebugMisc(arg1)	-- Unit casting the spell	(player)
	self:DebugMisc(arg2)	-- Spell name				(Captain Rumsey's Lager)
	self:DebugMisc(arg3)	-- Spell rank				()
	self:DebugMisc(arg4)	-- Spell lineID counter		(9)
	self:DebugMisc(arg5)	-- Spell ID					(45695)
end

function Capitalist:UNIT_SPELLCAST_SUCCEEDED(...)
	self:DebugEntry("UNIT_SPELLCAST_SUCCEEDED")

	local event, arg1, arg2, arg3, arg4, arg5 = ...;
	self:DebugMisc(arg1)	-- Unit casting the spell	(player)
	self:DebugMisc(arg2)	-- Spell name				(Captain Rumsey's Lager)
	self:DebugMisc(arg3)	-- Spell rank				()
	self:DebugMisc(arg4)	-- Spell lineID counter		(9)
	self:DebugMisc(arg5)	-- Spell ID					(45695)

	if (arg1 == "player") then
		local spellName = arg2
		local spellId = arg5

		self.skillCompleted = nil
		if (self.db.profile.professions) then
			local skillName = spellName
			-- REVIEW: Correction for two glyphs named "Glyph of Death Coil"
			if (skillName == "Glyph of Death Coil") then
				-- item 45804, spell 64266 - DK
				if (spellId == 64266) then
					skillName = skillName.." <Death Knight>"
				end
				-- item 42457, spell 57261 - Warlock
				if (spellId == 57261) then
					skillName = skillName.." <Warlock>"
				end
			end

			for profession, professionData in pairs(self.db.profile.professions) do
				local skillData = professionData[skillName]
				if (skillData) then
					local recipeLink = skillData.recipeLink
					if (recipeLink) then
						local recipeIdString = string.match(recipeLink, "Henchant:(%d+)")
						local recipeIdNumber = tonumber(recipeIdString)
						if (recipeIdNumber == arg5) then
							if (self.db.profile[realm][character][profession] and
							    self.db.profile[realm][character][profession][skillName]) then
								local numCraftOld = (self.db.profile[realm][character][profession][skillName].numCraft or 0)
								self.db.profile[realm][character][profession][skillName].numCraft = numCraftOld + 1
								self.skillCompleted = { profession = profession, skillName = skillName, }
								self:UpdateStatistics()
								AceConfigRegistry:NotifyChange("Capitalist")
								break
							end
						end
					end
				end
			end
		end
	end

	self:DebugExit("UNIT_SPELLCAST_SUCCEEDED")
end

function Capitalist:UPDATE_TRADESKILL_RECAST(...)
	self:DebugEntry("UPDATE_TRADESKILL_RECAST")
	local event, arg1, arg2, arg3, arg4, arg5 = ...;
	self:DebugMisc(arg1)
	self:DebugMisc(arg2)
	self:DebugMisc(arg3)
	self:DebugMisc(arg4)
	self:DebugMisc(arg5)
	self:DebugExit("UPDATE_TRADESKILL_RECAST")
end
